#include "../common/cleanup.h"
#include "../common/const.h"
#include "../common/global.h"
#include "../common/tls.h"
#include <arpa/inet.h>
#include <csignal>
#include <iostream>
#include <openssl/err.h>
#include <openssl/ssl.h>
#include <sys/socket.h>
#include <unistd.h>
SSL_CTX *ctx;
int sockfd;

int main() {
  std::signal(SIGTERM, safe_exit);
  std::signal(SIGINT, safe_exit);
  std::signal(SIGQUIT, safe_exit);
  std::signal(SIGHUP, safe_exit);

  // error signal handlers
  signal(SIGSEGV, safe_exit);

  ctx = create_context(true);
  sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if (sockfd < 0) {
    std::cerr << "Socket creation failed: " << strerror(errno) << std::endl;
    return 1;
  }
  std::cout << "Socket created successfully (fd: " << sockfd << ")"
            << std::endl;

  sockaddr_in addr{};
  addr.sin_family = AF_INET;
  addr.sin_port = htons(PORT);
  addr.sin_addr.s_addr = INADDR_ANY;

  if (bind(sockfd, (sockaddr *)&addr, sizeof(addr)) != 0) {
    std::cerr << "Bind failed: " << strerror(errno) << std::endl;
    return 1;
  }
  listen(sockfd, 5);

  while (true) {
    sockaddr_in client_addr{};
    socklen_t len = sizeof(client_addr);
    int client_fd = accept(sockfd, (sockaddr *)&client_addr, &len);

    SSL *ssl = SSL_new(ctx);
    SSL_set_fd(ssl, client_fd);
    SSL_accept(ssl);

    handle_connection(ssl);

    SSL_shutdown(ssl);
    SSL_free(ssl);
    close(client_fd);
  }

  SSL_CTX_free(ctx);
  close(sockfd);
  return 0;
}
